ALTER TABLE call_recording ENGINE=InnoDB;
ALTER TABLE call_recording ADD FOREIGN KEY (id_call_incoming) REFERENCES call_entry(id);
ALTER TABLE call_recording ADD FOREIGN KEY (id_call_outgoing) REFERENCES calls(id);

/* Tabla que define campañas de marcación manual */
CREATE TABLE IF NOT EXISTS manualdialing_campaign
(
    id                      INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    name                    VARCHAR(64)         NOT NULL,

    /* Fecha y hora de validez de campaña manual */
    datetime_init           DATE                NOT NULL,
    datetime_end            DATE                NOT NULL,
    daytime_init            TIME                NOT NULL,
    daytime_end             TIME                NOT NULL,

    /* NULL para marcado a través de dialplan, o troncal a usar */
    trunk                   VARCHAR(255),

    /* Contexto de dialplan dentro del cual iniciar marcado, normalmente from-internal */
    context                 VARCHAR(32)         NOT NULL,

    /* Texto inicial asociado a toda la campaña */
    script                  TEXT                NOT NULL,

    /* Estado de campaña: (A)ctiva, (I)nactiva, (T)erminada */
    estatus                 ENUM('A', 'I', 'T') NOT NULL    DEFAULT 'A',

    id_url                  INT(10) UNSIGNED ,  /* ID de url opcional */
    id_team                 INT(10) UNSIGNED,   /* ID de equipo de supervisión */

    /* El agente siempre debe poder saltarse un contacto. El escenario más
     * probable es aquel en el cual ninguno de los números del contacto
     * contesta.
     *
     * Comportamiento permitido al agente con la lista de llamadas. Los
     * comportamientos definidos hasta ahora son los siguientes:
     *
     * random-seek              El agente puede elegir cualquier llamada autorizada
     *                          al azar y marcarla como saltada, sin restricción.
     * sequential-skip          El agente sólo puede revisar llamadas de forma
     *                          secuencial, pero puede marcarlas como saltadas
     *                          sin haberlas marcado.
     * secuential-skip-tried    El agente sólo puede revisar llamadas de forma
     *                          secuencial, y está además obligado a intentar
     *                          marcar al menos un número del contacto antes de
     *                          poder marcarla como saltada y revisar la siguiente.
     *                          TODO: obligar a marcar todos los números del contacto?
     */
    call_seek_behavior      ENUM('random-seek', 'sequential-skip', 'sequential-skip-tried') NOT NULL DEFAULT 'random-seek',


    /* Plantilla opcional que describe de forma corta a un contacto en términos
     * de los atributos asociados. Esta plantilla es una cadena de texto
     * arbitraria, en la cual {NombreCampo} se reemplaza por el texto del
     * atributo cuyo nombre de columna sea NombreCampo. Además de estos
     * atributos, se dispone de {__CAMPAIGN_ID__} y {__CONTACT_ID__}. Si se deja
     * en NULL, la plantilla equivalente será: contact-{__CAMPAIGN_ID__}-{__CONTACT_ID__} */
    contact_description_template VARCHAR(255),

    PRIMARY KEY (id),
    FOREIGN KEY (id_url) REFERENCES campaign_external_url(id),
    FOREIGN KEY (id_team) REFERENCES team (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que define cuáles agentes atenderán las llamadas de esta campaña
 * manual. Necesario porque esta campaña no usa colas. */
CREATE TABLE IF NOT EXISTS manualdialing_campaign_agent
(
    id_campaign             INT(10) UNSIGNED    NOT NULL,
    id_agent                INT(10) UNSIGNED    NOT NULL,

    FOREIGN KEY (id_campaign) REFERENCES manualdialing_campaign (id),
    FOREIGN KEY (id_agent) REFERENCES agent (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que define una persona a contactar a través de marcación manual */
CREATE TABLE IF NOT EXISTS manualdialing_contact
(
    id                      INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_campaign             INT(10) UNSIGNED    NOT NULL,

    /* ID del agente que debe de atender este contacto, o NULL si cualquiera
     * puede atenderlo */
    id_agent                INT(10) UNSIGNED,

    /* Estado de procesamiento del contacto:
        pending     El contacto está pendiente de ser contactado.
        reserved    El contacto ha sido reservado por un agente, quien está
                    leyendo los datos del contacto y decidiendo si atender o
                    no la llamada. O bien ya está marcando o atendiendo al cliente.
        finished    El contacto ya fue procesado por un agente, el cual dio por
                    finalizado el trato con el contacto. Este estado sólo se debe
                    poder llegar luego de haber realizado al menos una llamada
                    exitosamente a un número del contacto, y sólo por el agente
                    que realizó esa llamada.
        Transiciones autorizadas:
        pending->reserved
        reserved->pending
        reserved->finished
     */
    processing_status       ENUM('pending','reserved','finished') NOT NULL DEFAULT 'pending',

    /* TODO: decidir si los siguientes dos campos merecen guardarse en historial */
    id_agent_reservation    INT(10) UNSIGNED,   /* ID del agente que hizo la reserva del contacto */
    datetime_reservation    DATETIME,           /* Instante en el que se hizo la reserva del contacto */

    PRIMARY KEY (id),
    FOREIGN KEY (id_campaign) REFERENCES manualdialing_campaign (id),
    FOREIGN KEY (id_agent) REFERENCES agent (id),
    FOREIGN KEY (id_agent_reservation) REFERENCES agent (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que define los atributos asociados con el contacto de marcación manual */
CREATE TABLE IF NOT EXISTS manualdialing_contact_attribute
(
    id              INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_contact      INT(10) UNSIGNED    NOT NULL,
    columna         VARCHAR(30)         NOT NULL,
    `value`         VARCHAR(128)        NOT NULL,
    column_number   INT UNSIGNED        NOT NULL    DEFAULT 0,

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES manualdialing_contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que define qué agentes se saltaron qué contactos. Esta tabla es
 * necesaria, porque un contacto que fue saltado por un agente y puede ser
 * atendido por cualquiera puede ser luego tomado por otro agente sin tener que
 * contactar al primer agente. */
CREATE TABLE IF NOT EXISTS manualdialing_contact_skipped
(
    id_agent        INT(10) UNSIGNED    NOT NULL,
    id_contact      INT(10) UNSIGNED    NOT NULL,

    FOREIGN KEY (id_agent) REFERENCES agent (id),
    FOREIGN KEY (id_contact) REFERENCES manualdialing_contact(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que define uno de los (múltiples) números asociados a un contacto de
 * marcación manual */
CREATE TABLE IF NOT EXISTS manualdialing_contact_number
(
    id              INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_contact      INT(10) UNSIGNED    NOT NULL,

    phone           VARCHAR(32)         NOT NULL,

    /* Esta bandera se setea a 1 si al momento de realizar la llamada el número
     * consta en la lista de DNC. */
    dnc             TINYINT(1)          NOT NULL    DEFAULT 0,

    /* NULL si número se cargó de CSV, o ID de agente que creó el número */
    id_agent_create INT(10) UNSIGNED,

    /* Rango de fechas y horas en las que cliente indicó que puede recibir
     * llamada. Si los 4 campos están en NULL, se asume ninguna preferencia en
     * particular.
     * TODO: vale la pena expandir en un calendario entero? */
    date_init       DATE,
    date_end        DATE,
    time_init       TIME,
    time_end        TIME,

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES manualdialing_contact(id),
    FOREIGN KEY (id_agent_create) REFERENCES agent (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que registra un intento de llamar al contacto de marcación manual a
 * través de un número específico. */
CREATE TABLE IF NOT EXISTS manualdialing_contact_number_callattempt
(
    id                          INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_contact_number           INT(10) UNSIGNED    NOT NULL,

    /* Los estados OnQueue y Abandoned requieren de una cola */
    status                      ENUM('Placing', 'Ringing', 'Success', 'Failure', 'NoAnswer', 'ShortCall') NOT NULL,

    /* Se marca en 1 si la llamada está en hold */
    onhold                      TINYINT(1)          NOT NULL    DEFAULT 0,

    uniqueid                    VARCHAR(32), /* Identificador asignado por Asterisk */
    trunk                       VARCHAR(20), /* Troncal por la cual salió la llamada */

    /* ID del agente responsable de la atención. Para marcación manual, es no-nulo */
    id_agent                    INT(10) UNSIGNED    NOT NULL,

    /* Instantes de eventos del intento de llamada */
    datetime_originate_start    DATETIME,   /* datetime_originate: se realizó el Originate */
    datetime_originate_end      DATETIME,   /* fecha_llamada: se recibió respuesta del Originate */
    /* datetime_entry_queue requiere de cola */
    datetime_link_start         DATETIME,   /* start_time: fecha de inicio de atención del agente */
    datetime_link_end           DATETIME,   /* end_time: fecha de final de atención o de proceso */

    /* duration_wait requiere de cola */
    duration                    INT(10) UNSIGNED,   /* duración asignada a la llamada */

    /* Causa reportada por Asterisk de intento fallido de marcación */
    failure_cause               INT(10) UNSIGNED,
    failure_cause_txt           VARCHAR(32),

    amd_status                  VARCHAR(12), /* estado de AMD, si está disponible */

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact_number) REFERENCES manualdialing_contact_number(id),
    FOREIGN KEY (id_agent) REFERENCES agent (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que almacena qué formularios estáticos deben usarse con la campaña manual */
CREATE TABLE IF NOT EXISTS campaign_form_manualdialing
(
    id_campaign     INT(10) UNSIGNED    NOT NULL,
    id_form         INT(10) UNSIGNED    NOT NULL,

    PRIMARY KEY (id_campaign, id_form),
    FOREIGN KEY (id_campaign) REFERENCES manualdialing_campaign (id),
    FOREIGN KEY (id_form) REFERENCES form(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que almacena los datos recogidos para el contacto con formulario estático */
CREATE TABLE IF NOT EXISTS form_data_recolected_manualdialing
(
    id              INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_contact      INT(10) UNSIGNED    NOT NULL,
    id_form_field   INT(10) UNSIGNED    NOT NULL,
    `value`         VARCHAR(250)        NOT NULL,

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES manualdialing_contact(id),
    FOREIGN KEY (id_form_field) REFERENCES form_field (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que almacena qué formularios dinámicos deben usarse con la campaña manual */
CREATE TABLE IF NOT EXISTS campaign_dynamic_form_manualdialing
(
    id_campaign     INT(10) UNSIGNED    NOT NULL,
    id_dynamic_form INT(10) UNSIGNED    NOT NULL,

    PRIMARY KEY (id_campaign, id_dynamic_form),
    FOREIGN KEY (id_campaign) REFERENCES manualdialing_campaign (id),
    FOREIGN KEY (id_dynamic_form) REFERENCES dynamic_form(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Tabla que almacena los datos recogidos para el contacto con formulario dinámico */
CREATE TABLE IF NOT EXISTS dynamic_form_data_recolected_manualdialing
(
    id                          INT(10) UNSIGNED    NOT NULL    AUTO_INCREMENT,
    id_contact                  INT(10) UNSIGNED    NOT NULL,
    id_dynamic_form_node_field  INT(10) UNSIGNED    NOT NULL,
    `value`                     TEXT                NOT NULL,

    PRIMARY KEY (id),
    FOREIGN KEY (id_contact) REFERENCES manualdialing_contact(id),
    FOREIGN KEY (id_dynamic_form_node_field) REFERENCES dynamic_form_node_field (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

/* Log de progreso de marcación */
ALTER TABLE call_progress_log
    ADD COLUMN id_campaign_manualdialing INT UNSIGNED AFTER id_call_outgoing,
    ADD FOREIGN KEY (id_campaign_manualdialing) REFERENCES manualdialing_campaign (id),
    ADD COLUMN id_call_manualdialing INT UNSIGNED AFTER id_campaign_manualdialing,
    ADD FOREIGN KEY (id_call_manualdialing) REFERENCES manualdialing_contact_number_callattempt (id);

/* Seguimiento de grabaciones */
ALTER TABLE call_recording
    ADD COLUMN id_call_manualdialing INT UNSIGNED,
    ADD FOREIGN KEY (id_call_manualdialing) REFERENCES manualdialing_contact_number_callattempt (id);

/* Actualización al nuevo uso de group_status. Esto asume que el dialer no
 * está ejecutándose en este momento. */
UPDATE calls SET group_status = NULL WHERE group_status <> 'Success';
UPDATE calls SET group_status = 'Finished' WHERE group_status = 'Success';
